@@ -1,19 +1,28 @@
 import os
-import urllib
+import codecs
 from collections import namedtuple
 from posixpath import normpath
 
+try:
+    from urllib.request import urlopen
+    from urllib.parse import quote
+    unicode = str
+except:
+    from urllib import urlopen
+    from urllib import quote
 
-__version__ = '0.2.2'
+
+__version__ = '0.3.1'
 
 __all__ = ['URL', 'SplitResult', 'parse', 'extract', 'construct', 'normalize',
-           'normalize_host', 'normalize_path', 'normalize_query',
+           'compare', 'normalize_host', 'normalize_path', 'normalize_query',
            'normalize_fragment', 'encode', 'unquote', 'split', 'split_netloc',
            'split_host']
 
 
 PSL_URL = 'https://publicsuffix.org/list/effective_tld_names.dat'
 
+
 def _get_public_suffix_list():
     """Return a set containing all Public Suffixes.
 
@@ -23,10 +32,10 @@ def _get_public_suffix_list():
     """
     local_psl = os.environ.get('PUBLIC_SUFFIX_LIST')
     if local_psl:
-        with open(local_psl) as f:
+        with codecs.open(local_psl, 'r', 'utf-8') as f:
             psl_raw = f.readlines()
     else:
-        psl_raw = urllib.urlopen(PSL_URL).readlines()
+        psl_raw = unicode(urlopen(PSL_URL).read(), 'utf-8').split('\n')
     psl = set()
     for line in psl_raw:
         item = line.strip()
@@ -48,7 +57,8 @@ DEFAULT_PORT = {
     'ws': '80',
     'wss': '443',
     'ftp': '21',
-    'sftp': '22'
+    'sftp': '22',
+    'ldap': '389'
 }
 QUOTE_EXCEPTIONS = {
     'path': ' /?+#',
@@ -63,17 +73,6 @@ URL = namedtuple('URL', ['scheme', 'username', 'password', 'subdomain',
                          'domain', 'tld', 'port', 'path', 'query', 'fragment',
                          'url'])
 
-_hextochr = {'%02x' % i: chr(i) for i in range(256)}
-_hextochr.update({'%02X' % i: chr(i) for i in range(256)})
-
-
-def _idna_encode(x):
-    return x.decode('utf-8').encode('idna')
-
-
-def _idna_decode(x):
-    return x.decode('idna').encode('utf-8')
-
 
 def normalize(url):
     """Normalize a URL.
@@ -81,15 +80,17 @@ def normalize(url):
     >>> normalize('hTtp://ExAMPLe.COM:80')
     'http://example.com/'
     """
-    if url.strip() == '':
+    url = url.strip()
+    if url == '':
         return ''
-    parts = split(url.strip())
+    parts = split(url)
     if parts.scheme:
         netloc = parts.netloc
         if parts.scheme in SCHEMES:
             path = normalize_path(parts.path)
         else:
             path = parts.path
+    # url is relative, netloc (if present) is part of path
     else:
         netloc = parts.path
         path = ''
@@ -105,6 +106,30 @@ def normalize(url):
                          port, path, query, fragment, None))
 
 
+def compare(url1, url2):
+    """Check if url1 and url2 are the same after normalizing both.
+
+    >>> compare("http://examPLe.com:80/abc?x=&b=1", "http://eXAmple.com/abc?b=1")
+    True
+    """
+    return normalize(url1) == normalize(url2)
+
+
+def _idna_encode(x):
+    return x.encode('idna').decode('utf-8')
+
+
+def _encode_query(query):
+    """Quote all values of a query string."""
+    if query == '':
+        return query
+    query_args = []
+    for query_kv in query.split('&'):
+        k, v = query_kv.split('=')
+        query_args.append(k + "=" + quote(v.encode('utf-8')))
+    return '&'.join(query_args)
+
+
 def encode(url):
     """Encode URL."""
     parts = extract(url)
@@ -115,9 +140,9 @@ def encode(url):
                          _idna_encode(parts.domain),
                          _idna_encode(parts.tld),
                          parts.port,
-                         urllib.quote(parts.path),
-                         urllib.quote(parts.query),
-                         urllib.quote(parts.fragment),
+                         quote(parts.path.encode('utf-8')),
+                         _encode_query(parts.query),
+                         quote(parts.fragment.encode('utf-8')),
                          None))
 
 
@@ -149,6 +174,10 @@ def construct(parts):
     return nurl
 
 
+def _idna_decode(x):
+    return codecs.decode(x.encode('utf-8'), 'idna')
+
+
 def normalize_host(host):
     """Normalize host (decode IDNA)."""
     if 'xn--' not in host:
@@ -164,7 +193,6 @@ def _normalize_port(scheme, port):
     >>> _normalize_port('http', '8080')
     '8080'
     """
-    assert type(port) == str
     if not scheme:
         return port
     if port and port != DEFAULT_PORT[scheme]:
@@ -206,10 +234,14 @@ def normalize_query(query):
 
 
 def normalize_fragment(fragment):
-    """Normalize fragment (unquote with exceptions only)"""
+    """Normalize fragment (unquote with exceptions only)."""
     return unquote(fragment, QUOTE_EXCEPTIONS['fragment'])
 
 
+_hextochr = {'%02x' % i: chr(i) for i in range(256)}
+_hextochr.update({'%02X' % i: chr(i) for i in range(256)})
+
+
 def unquote(text, exceptions=[]):
     """Unquote a text but ignore the exceptions.
 
@@ -338,17 +370,15 @@ def split(url):
 
 
 def _clean_netloc(netloc):
-    """Remove trailing '.' and ':' and tolower
+    """Remove trailing '.' and ':' and tolower.
 
     >>> _clean_netloc('eXample.coM:')
     'example.com'
     """
     try:
-        netloc.encode('ascii')
+        return netloc.rstrip('.:').lower()
     except:
         return netloc.rstrip('.:').decode('utf-8').lower().encode('utf-8')
-    else:
-        return netloc.rstrip('.:').lower()
 
 
 def split_netloc(netloc):
